@@ -98,7 +98,7 @@ IOService * IOBacklightDisplay::probe( IOService * provider, SInt32 * score )
                     && (kPanelFSTNConnect != displayType))
                 continue;
 
-            OSIterator * iter = getMatchingServices( nameMatching("backlight") );
+            OSIterator * iter = getMatchingServices(nameMatching("backlight"));
             if (iter)
             {
                 haveBacklight = (0 != iter->getNextObject());
@@ -153,18 +153,23 @@ void IOBacklightDisplay::initPowerManagement( IOService * provider )
         fMaxBrightness = 255;
         fCurrentBrightness = fMaxBrightness;
     }
-    if (displayParams)
-	displayParams->release();
 
     if (fCurrentBrightness < (fMinBrightness + 2))
         fCurrentBrightness = fMinBrightness + 2;
 
     if ((num = OSDynamicCast(OSNumber,
                              getConnection()->getFramebuffer()->getProperty(kIOBacklightUserBrightnessKey))))
+    {
         fCurrentUserBrightness = num->unsigned32BitValue();
+	if (displayParams)
+	    setParameter(displayParams, gIODisplayBrightnessKey, fCurrentUserBrightness);
+    }
     else
         fCurrentUserBrightness = fCurrentBrightness;
 
+    if (displayParams)
+	displayParams->release();
+
     fMaxBrightnessLevel[0] = 0;
     fMaxBrightnessLevel[1] = fMinBrightness;
     fMaxBrightnessLevel[2] = fMinBrightness + 1;
@@ -172,6 +177,8 @@ void IOBacklightDisplay::initPowerManagement( IOService * provider )
 
     fDisplayPMVars->currentState = kIODisplayMaxPowerState;
 
+    getPMRootDomain()->publishFeature("DisplayDims");
+
     // initialize superclass variables
     PMinit();
     // attach into the power management hierarchy
@@ -181,6 +188,39 @@ void IOBacklightDisplay::initPowerManagement( IOService * provider )
     registerPowerDriver(this, (IOPMPowerState *) ourPowerStates, kIODisplayNumPowerStates);
 }
 
+IOReturn IOBacklightDisplay::setAggressiveness( unsigned long type, unsigned long newLevel )
+{
+    if (type == (unsigned long) kIODisplayDimAggressiveness)
+    {
+        newLevel = newLevel ? (fMinBrightness + 1) : fMaxBrightness;
+        if (newLevel != fMaxBrightnessLevel[2])
+        {
+            fMaxBrightnessLevel[2] = newLevel;
+            if (2 == fCurrentPowerState)
+                setBrightness(newLevel);
+        }
+    }
+
+    super::setAggressiveness(type, newLevel);
+
+    return (kIOReturnSuccess);
+}
+
+IOReturn IOBacklightDisplay::getAggressiveness( unsigned long type, unsigned long * currentLevel )
+{
+    IOReturn ret;
+
+    if (type == (unsigned long) kIODisplayDimAggressiveness)
+    {
+	*currentLevel = (fMaxBrightnessLevel[2] == (fMinBrightness + 1));
+	ret = kIOReturnSuccess;
+    }
+    else
+	ret = super::getAggressiveness(type, currentLevel);
+
+    return (ret);
+}
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 // setPowerState
 //
@@ -360,7 +400,6 @@ class AppleBacklightDisplay : public IOBacklightDisplay
     OSDeclareDefaultStructors(AppleBacklightDisplay)
 
 protected:
-    IONotifier *	     fNotify;
     IOInterruptEventSource * fDeferredEvents;
 
 public:
@@ -373,8 +412,6 @@ public:
                                        IOIndex event, void * info );
 
 private:
-    static bool _clamshellHandler( void * target, void * ref,
-                                   IOService * resourceService );
     static void _deferredEvent( OSObject * target,
                                 IOInterruptEventSource * evtSrc, int intCount );
 };
@@ -393,14 +430,11 @@ bool AppleBacklightDisplay::start( IOService * provider )
     if (!super::start(provider))
         return (false);
 
-    fNotify = addNotification( gIOPublishNotification,
-                               resourceMatching(kAppleClamshellStateKey),
-                               _clamshellHandler, this, 0, 10000 );
-
     fDeferredEvents = IOInterruptEventSource::interruptEventSource(this, _deferredEvent);
     if (fDeferredEvents)
         getWorkLoop()->addEventSource(fDeferredEvents);
 
+    getConnection()->getFramebuffer()->setProperty(kIOFBBuiltInKey, this, 0);
 
 
     return (true);
@@ -410,20 +444,12 @@ void AppleBacklightDisplay::stop( IOService * provider )
 {
     IOFramebuffer::clamshellEnable( -1 );
 
-    getPMRootDomain()->removeProperty(kAppleClamshellStateKey);
-
     if (fDeferredEvents)
     {
         getWorkLoop()->removeEventSource(fDeferredEvents);
         fDeferredEvents = 0;
     }
 
-    if (fNotify)
-    {
-        fNotify->remove();
-        fNotify = 0;
-    }
-
     getConnection()->getFramebuffer()->setProperty(kIOBacklightUserBrightnessKey,
             fCurrentUserBrightness, 32);
 
@@ -466,55 +492,23 @@ IOReturn AppleBacklightDisplay::framebufferEvent( IOFramebuffer * framebuffer,
 
         setBrightness( value );
     }
-
-    return (kIOReturnSuccess);
-}
-
-bool AppleBacklightDisplay::_clamshellHandler( void * target, void * ref,
-        IOService * resourceService )
-{
-    AppleBacklightDisplay * self = (AppleBacklightDisplay *) target;
-    UInt32		    value;
-    OSObject *		    prop = 0;
-    OSObject *		    clamshellProperty;
-
-
-    clamshellProperty = resourceService->getProperty(kAppleClamshellStateKey);
-    if (!clamshellProperty)
-        return (true);
-
-    value = (clamshellProperty == kOSBooleanTrue);
-    gIOFBLastClamshellState  = value;
-
-    //    if( !prop)        return( true );
-
-    getPMRootDomain()->setProperty(kAppleClamshellStateKey, clamshellProperty);
-    resourceService->removeProperty(kAppleClamshellStateKey);
-
-    if ((kOSBooleanTrue == prop) && (kIOPMEnableClamshell & IOFramebuffer::clamshellState()))
+    else if (kIOFBNotifyClamshellChange == event)
     {
-        if (value)
-        {
+	if ((kOSBooleanTrue == info) && (kIOPMEnableClamshell & IOFramebuffer::clamshellState()))
+	{
 #if LCM_HWSLEEP
-            self->fConnection->getFramebuffer()->changePowerStateTo(0);
+	    fConnection->getFramebuffer()->changePowerStateTo(0);
 #endif
-            self->changePowerStateToPriv( 0 );
-        }
-        else
-        {
-            self->changePowerStateToPriv( kIODisplayMaxPowerState );
-#if LCM_HWSLEEP
-            self->fConnection->getFramebuffer()->changePowerStateTo(1);
-#endif
-        }
+	    changePowerStateToPriv( 0 );
+	}
+
+	// may be in the right power state already, but wrong brightness because
+	// of the clamshell state at setPowerState time.
+	if (fDeferredEvents)
+	    fDeferredEvents->interruptOccurred(0, 0, 0);
     }
 
-    // may be in the right power state already, but wrong brightness because
-    // of the clamshell state at setPowerState time.
-    if (self->fDeferredEvents)
-        self->fDeferredEvents->interruptOccurred(0, 0, 0);
-
-    return (true);
+    return (super::framebufferEvent( framebuffer, event, info ));
 }
 
 bool AppleBacklightDisplay::setBrightness( SInt32 value )
